"""
Transformers for all the user setting events
"""
import json


def edx_user_settings_viewed(current_event, caliper_event):
    """
    Event occur when you go to your Account Settings Page.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_extensions = json.loads(current_event['event'])

    caliper_object = {
        'id': current_event['referer'],
        'type': 'WebPage',
        'extensions': object_extensions
    }

    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['extensions']['extra_fields'].pop('user_id')

    return caliper_event


def edx_user_settings_changed(current_event, caliper_event):
    """
    Event occurs when a user changes anything in his/her account settings page.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_extensions = current_event['event']

    caliper_object = {
        'id': caliper_event['actor']['id'],
        'type': 'Person',
        'extensions': object_extensions
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    return caliper_event


def edx_bi_course_upgrade_sidebarupsell_displayed(current_event, caliper_event):
    """
    Browser emits this event when sidebar for upgrading course is displayed on course page.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    object_extensions = json.loads(current_event['event'])

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Frame',
        'extensions': object_extensions
    }

    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    return caliper_event



"""
Transformers for all peer assessment related events.
"""


def ubc_peer_instruction_accessed(current_event, caliper_event):
    """
    The server emits this event when a peer instruction question
    and its set of answer choices is shown to a learner.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': {
            'id': current_event['referer'],
            'type': 'AssignableDigitalResource',
            'extensions': current_event['event']
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'asides': current_event['context']['asides'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags'],
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event


def ubc_peer_instruction_original_submitted(current_event, caliper_event):
    """
    The server emits this event when learners submit their initial responses.
    These events records the answer choice the learner selected and the
    explanation given for why that selection was made.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'AssessmentEvent',
        'action': 'Submitted',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
            'extensions': current_event['event']
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'asides': current_event['context']['asides'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'module': current_event['context']['module'],
        'ip': current_event['ip']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event


def ubc_peer_instruction_revised_submitted(current_event, caliper_event):
    """
    The server emits this event when the learners submit their revised responses.
    These events record the answer choice selected by the learner and the explanation
    for why that selection was made.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'AssessmentEvent',
        'action': 'Submitted',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
            'extensions': current_event['event']
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'asides': current_event['context']['asides'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'module': current_event['context']['module'],
        'ip': current_event['ip']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event



"""
Transformers for all the library content block events
"""
from django.conf import settings


def edx_librarycontentblock_content_removed(current_event, caliper_event):
    """
    The server emits an edx.librarycontentblock.content.removed event when a
    user revisits a randomized content block and one or more of the components
    that were previously delivered to that user can no longer be delivered.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'AssessmentItem',
        'extensions': current_event['event']
    }

    caliper_event.update({
        'action': 'Deleted',
        'type': 'Event',
        'object': caliper_object
    })
    caliper_event['referrer'].update({
        'type': 'WebPage'
    })
    caliper_event['actor'].update({
        'id': settings.LMS_ROOT_URL,
        'type': 'SoftwareApplication'
    })

    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    caliper_event['extensions']['extra_fields']['username'] = current_event['username']
    caliper_event['extensions']['extra_fields'].pop('session')

    return caliper_event


def edx_librarycontentblock_content_assigned(current_event, caliper_event):
    """
    The server emits an edx.librarycontentblock.content.assigned event the
    first time that content from a randomized content block is delivered to
    a user. The edx.librarycontentblock.content.assigned event identifies
    the components delivered from the library to a user.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    current_event_details = current_event['event']

    current_event_details.update({
        'learner': {
            'id': caliper_event['actor']['id'],
            'name': current_event['username'],
            'type': 'Person',
            'user_id': caliper_event['extensions']['extra_fields'][
                'user_id']
        }
    })

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Assessment',
        'extensions': current_event_details
    }

    caliper_event.update({
        'action': 'Linked',
        'type': 'Event',
        'object': caliper_object
    })

    caliper_event['actor'].update({
        'id': settings.LMS_ROOT_URL,
        'type': 'SoftwareApplication'
    })

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'ip': current_event['ip'],
        'username': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event



"""
Transformers for all Course Content Completion Events
"""
import json


def edx_done_toggled(current_event, caliper_event):
    """
    Both the browser and the server emit the edx.done.toggled event when the control added by the
    Completion XBlock is toggled. The event_source field indicates whether the event was emitted by
    the client or the server.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'action': 'Used',
        'type': 'ToolUseEvent',
        'object': {
            'id': current_event['referer'],
            'type': 'SoftwareApplication',
        }
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']

    if current_event.get('event_source') == 'server':
        caliper_event['extensions']['extra_fields'].pop('session')
        caliper_event['object']['extensions'] = current_event['event']
    else:
        caliper_event['object'].update({
            'extensions': json.loads(current_event['event'])
        })

    return caliper_event



"""
Transformers for all the bookmark events
"""
import json


def edx_bookmark_listed(current_event, caliper_event):
    """
    The server emits this event when a user clicks Bookmarks under
    the Course Tools heading in the LMS to view the list of previously
    bookmarked pages. If the number of bookmarks exceeds the defined
    page length, the browser emits an additional edx.course.bookmark.listed
    event each time the user navigates to a different page of results.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': {
            'id': current_event['referer'],
            'type': 'WebPage',
            'extensions': {
                'course_id': current_event['event'].get('course_id'),
                'page_number': current_event['event'].get('page_number'),
                'bookmarks_count': current_event['event'].get('bookmarks_count'),
                'page_size': current_event['event'].get('page_size'),
                'list_type': current_event['event'].get('list_type'),
            }
        }
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event.get('ip'),
    })

    return caliper_event


def edx_bookmark_added(current_event, caliper_event):
    """
    The server emits this event when a user removes a bookmark from a page.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_event.update(
        {
            'type': 'AnnotationEvent',
            'action': 'Bookmarked',
            'object': {
                'id': current_event['referer'],
                'type': 'Page',
                'extensions': {
                    'course_id': current_event['event']['course_id'],
                    'bookmark_id': current_event['event']['bookmark_id'],
                    'component_usage_id': current_event['event']['component_usage_id'],
                    'component_type': current_event['event']['component_type']
                }
            }
        }
    )
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'path': current_event['context']['path'],
    })
    return caliper_event


def edx_bookmark_accessed(current_event, caliper_event):
    """
    When bookmarks are accessed.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_event.update({
        'action': 'NavigatedTo',
        'type': 'NavigationEvent'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event.get('ip')
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['actor'].update({
        'name': current_event.get('username'),
        'type': 'Person'
    })
    event_info = json.loads(current_event['event'])
    caliper_event['object'] = {
        'id': current_event.get('referer'),
        'type': 'WebPage',
        'extensions': event_info
    }
    return caliper_event


def edx_bookmark_removed(current_event, caliper_event):
    """
    When a bookmark is removed/deleted.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_event.update({
        'action': 'Deleted',
        'type': 'Event'
    })
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event.get('ip'),
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['actor'].update({
        'name': current_event.get('username'),
        'type': 'Person'
    })
    caliper_event['object'] = {
        'id':  current_event.get('referer'),
        'type': 'BookmarkAnnotation',
        'extensions': current_event['event']
    }
    return caliper_event



"""
Transformers for all course related events.
"""

from django.conf import settings
from django.core.urlresolvers import reverse


def edx_course_enrollment_activated(current_event, caliper_event):
    """
    When a student enrolls in a course, the server emits an
    edx.course.enrollment.activated event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Activated',
        'object': {
            'id': caliper_event['referrer']['id'],
            'type': 'Membership',
            'extensions': {
                'mode': current_event['event']['mode'],
                'course_id': current_event['context']['course_id'],
                'user_id': caliper_event['extensions']['extra_fields'].pop(
                    'user_id')
            }
        },
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].update({
        'event_source': current_event['event_source'],
        'ip': current_event['ip'],

    })
    return caliper_event


def edx_course_enrollment_deactivated(current_event, caliper_event):
    """
    The server emits an enrollment.deactivated event when the student's
    enrollment is cancelled.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    course_id = current_event['context']['course_id']

    course_link = '{lms_url}{profile_link}'.format(
        lms_url=settings.LMS_ROOT_URL,
        profile_link=str(reverse(
            'about_course',
            kwargs={'course_id': course_id}
        ))
    )

    caliper_object = {
        'id': course_link,
        'type': 'Membership',
        'extensions': {
            'course_id': course_id,
            'mode': current_event['event']['mode']
        }
    }

    caliper_event.update({
        'object': caliper_object,
        'type': 'Event',
        'action': 'Deactivated',
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_course_enrollment_mode_changed(current_event, caliper_event):
    """
    The server emits an enrollment.mode_changed event when the student's
    enrollment mode is changed.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': {
            'id': current_event['referer'],
            'type': 'Membership',
            'extensions': {
                'course_id': current_event['context']['course_id'],
                'org_id': current_event['context']['org_id'],
                'mode': current_event['event']['mode'],
            }
        },
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip']
    })

    return caliper_event


def edx_course_enrollment_upgrade_clicked(current_event, caliper_event):
    """
    The server emits an enrollment.upgrade_clicked event when the student
    clicks on Upgrade Enrollment Button.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    caliper_object = {
        'id': current_event['page'],
        'type': 'WebPage',
    }

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': caliper_object,

    })

    caliper_event['extensions']['extra_fields'].update({
        'event': current_event['event'],
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    return caliper_event


def edx_course_enrollment_upgrade_succeeded(current_event, caliper_event):
    """
    This event is generated when the user is successfully upgraded, from honor
    to verified or so, in a course.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': {
            'id': caliper_event['referrer']['id'],
            'type': 'Membership',
            'extensions': {
                'mode': current_event['event']['mode'],
                'course_id': current_event['context']['course_id'],
                'user_id': caliper_event['extensions']['extra_fields']
                    .pop('user_id'),
                'org_id': caliper_event['extensions']['extra_fields']
                    .pop('org_id')
            }
        },
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].update({
        'event_source': current_event['event_source'],
        'ip': current_event['ip'],

    })
    return caliper_event



"""
Transformers for all the drag and drop events
"""


def edx_drag_and_drop_v2_item_dropped(current_event, caliper_event):
    """
    The server emits this event when a learner releases a draggable item
    into a target zone in a drag and drop problem.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    caliper_object = {
        'id': current_event['referer'],
        'extensions': current_event['event'],
        'type': 'AssessmentItem'
    }

    caliper_event.update({
        'action': 'Completed',
        'type': 'AssessmentItemEvent',
        'object': caliper_object
    })

    caliper_event['referrer'].update({
        'id': current_event['referer'],
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update(
        current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event


def edx_drag_and_drop_v2_item_picked_up(current_event, caliper_event):
    """
    The server emits this event when a learner selects a draggable item in a
    drag and drop problem.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    caliper_object = {
        'id': current_event['referer'],
        'extensions': current_event['event'],
        'type': 'AssessmentItem'
    }

    caliper_event.update({
        'action': 'Started',
        'type': 'AssessmentItemEvent',
        'object': caliper_object
    })

    caliper_event['referrer'].update({
        'id': current_event['referer'],
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update(
        current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event


def edx_drag_and_drop_v2_loaded(current_event, caliper_event):
    """
    The server emits this event after a drag and drop problem is shown in the LMS.


    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    caliper_object = {
        'id': current_event['referer'],
        'type': 'AssessmentItem',
        'extensions': current_event['event']
    }

    caliper_event.update({
        'action': 'Viewed',
        'type': 'ViewEvent',
        'object': caliper_object
    })

    caliper_event['referrer'].update({
        'id': current_event['referer'],
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update(
        current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event


def edx_drag_and_drop_v2_feedback_opened(current_event, caliper_event):
    """
    The server emits this event when a pop up feedback message opens in
     a drag and drop problem.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_extensions = current_event['context']

    object_extensions.update(current_event['event'])

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Frame',
        'extensions': object_extensions
    }

    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'path': caliper_event['object']['extensions'].pop('path')
    })

    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['object']['extensions'].pop('user_id')

    return caliper_event


def edx_drag_and_drop_v2_feedback_closed(current_event, caliper_event):
    """
    The server emits this event when a pop up
    feedback message closes in a drag and drop problem.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_extensions = current_event['context']

    object_extensions.update(current_event['event'])

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Frame',
        'extensions': object_extensions
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Removed',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'path': caliper_event['object']['extensions'].pop('path')
    })

    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['object']['extensions'].pop('user_id')

    return caliper_event



import os
import json
import requests
from datetime import datetime

from loggers import get_test_logger, get_formatted_log
from settings import CALIPER_AUTH

CALIPER_TESTS_API_URL = CALIPER_AUTH.get('CALIPER_TESTS_API_URL')
CALIPER_TESTS_API_TOKEN = CALIPER_AUTH.get('CALIPER_TESTS_API_TOKEN')


TEST_DIR_PATH = 'tests'
TESTS_LOG_FILE = 'caliper_tests.log'


logger = get_test_logger('caliper_tests', TESTS_LOG_FILE)


test_files = [
    file for file in os.listdir(
        '{}/expected/'.format(TEST_DIR_PATH)
    ) if file.endswith(".json")
]
for file in test_files:
    test_json = json.load(
        open('{}/expected/{}'.format(TEST_DIR_PATH, file), 'r'))
    if not '@context' in test_json:
        continue
    TEST_ENVELOP = {
        "sensor": "https://example.edu/sensors/1",
        "sendTime": str(datetime.now())[:-3] + 'Z',
        "dataVersion": "http://purl.imsglobal.org/ctx/caliper/v1p1",
        "data": [test_json]
    }
    BEARER_TOKEN = 'Bearer {}'.format(CALIPER_TESTS_API_TOKEN)
    headers = {
        'Authorization': BEARER_TOKEN,
        'Content-Type': "application/json",
    }
    response = requests.request('POST',
                                CALIPER_TESTS_API_URL,
                                data=json.dumps(TEST_ENVELOP),
                                headers=headers
                                )
    if response.status_code == requests.codes.ok:
        logger.info(
            get_formatted_log(
                'OK', file, test_json['extensions']['extra_fields']['event_type'], response.status_code)
        )
    else:
        logger.error(
            get_formatted_log(
                'ERROR', file, test_json['extensions']['extra_fields']['event_type'], response.status_code)
        )





"""
Transformers for all the certificate events
"""

from openedx_caliper_tracking.utils import get_certificate_url
from django.conf import settings

import json


def edx_certificate_evidence_visited(current_event, caliper_event):
    """
    When a learner shares her certificates on social network sites such
    as LinkedIn, and the link back to the certificate is followed by some
    visitor to that social network site, the server emits an
    edx.certificate.evidence_visited event.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    user_id = current_event['event'].get('user_id')
    course_id = current_event['event'].get('course_id')

    certificate_uri = get_certificate_url(user_id, course_id)

    object_extensions = current_event['event']

    caliper_event.update({
        'type': 'Event',
        'action': 'Showed',
        'object': {
            'id': certificate_uri,
            'type': 'Document',
            'extensions': object_extensions
        }
    })

    caliper_event['actor'].update({
        'id': settings.LMS_ROOT_URL,
        'type': 'SoftwareApplication'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event.get('ip'),
        'referer': current_event.get('referer'),
        'username': current_event.get('username')
    })

    caliper_event['extensions']['extra_fields'].update(
        current_event['context']
    )

    caliper_event.pop('referrer')

    return caliper_event


def edx_certificate_shared(current_event, caliper_event):
    """
    When a learner shares the URL for her certificate on a social media
    web site, the server emits an edx.certificate.shared event.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_extensions = json.loads(current_event['event'])

    caliper_event.update({
        'type': 'Event',
        'action': 'Shared',
        'object': {
            'id': object_extensions['certificate_url'],
            'type': 'Document',
            'extensions': object_extensions
        }
    })

    caliper_event['actor'].update({
        'type': 'Person'
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event.get('ip'),
        'course_id': current_event['context'].get('course_id'),
    })

    return caliper_event


def edx_certificate_created(current_event, caliper_event):
    """
    When a certificate is generated, a record is created in the
    certificates_generatedcertificate table, triggering an
    edx.certificate.created event.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    user_id = current_event['context'].get('user_id')
    course_id = current_event['context'].get('course_id')

    certificate_uri = get_certificate_url(user_id, course_id)

    object_extensions = current_event['event']

    caliper_event.update({
        'type': 'Event',
        'action': 'Created',
        'object': {
            'id': certificate_uri,
            'type': 'Document',
            'extensions': object_extensions
        }
    })

    caliper_event['actor'].update({
        'id': settings.LMS_ROOT_URL,
        'type': 'SoftwareApplication'
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event.get('ip'),
        'course_id': current_event['context'].get('course_id'),
        'course_user_tags': current_event['context'].get('course_user_tags'),
        'username': current_event.get('username')
    })

    return caliper_event



"""
Transformers for all the course settings related events
"""

import json


def edx_course_home_resume_course_clicked(current_event, caliper_event):
    """
    This event is generated when we press the start course button on a course

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_extensions = json.loads(current_event['event'])

    object_extensions.update({
        'course_id': current_event['context']['course_id'],
        'org_id': caliper_event['extensions']['extra_fields'].pop('org_id'),
    })

    caliper_object = {
        'id': object_extensions.pop('url'),
        'type': 'CourseSection',
        'extensions': object_extensions
    }

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
    })

    return caliper_event


def edx_grades_grading_policy_changed(current_event, caliper_event):
    """
    Event occur while adding grades policy for entrance in course

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_object = {
        'id': current_event['referer'],
        'type': 'CourseOffering',
        'extensions': current_event['event']
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['event']['course_id']
    })

    return caliper_event



"""
Transformers for all the team events
"""

import json
from openedx_caliper_tracking import utils


def edx_team_page_viewed(current_event, caliper_event):
    """
    When a user views any page with a unique URL under the Teams page in the
    courseware, the browser emits an edx.team.page_viewed event.
    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    current_event_details = json.loads(current_event['event'])

    caliper_object = {
        'id': current_event['page'],
        'type': 'WebPage',
        'extensions': current_event_details
    }

    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': caliper_object
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_team_learner_added(current_event, caliper_event):
    """
    When a user joins a team or is added by someone else,
    the server emits an edx.team.learner_added event.
    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    username = utils.get_username_from_user_id(
        current_event['event']['user_id'])

    user_link = utils.get_user_link_from_username(username)

    object_link = utils.get_team_url_from_team_id(
        current_event['referer'],
        current_event['event']['team_id']
    )

    caliper_object = {
        'id': object_link,
        'member': {
            'extensions': {
                'user_id': current_event['event']['user_id']
            },
            'id': user_link,
            'name': username,
            'type': 'Person'
        },
        'organization': {
            'extensions': {
                'team_id': current_event['event']['team_id']
            },
            'id': object_link,
            'type': 'Group'
        },
        'type': 'Membership',
        'extensions': {
            'add_method': current_event['event']['add_method']
        }
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Added',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
    })

    return caliper_event


def edx_team_deleted(current_event, caliper_event):
    """
    This event is generated when new team is deleted

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    team_link = current_event['referer']

    caliper_object = {
        'extensions': current_event['event'],
        'id': team_link,
        'type': 'Group'
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Deleted',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    return caliper_event


def edx_team_changed(current_event, caliper_event):
    """
    This event is generated when user edits any field in a team.
    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_extensions = current_event['event']

    object_link = utils.get_team_url_from_team_id(
        current_event['referer'],
        current_event['event']['team_id']
    )

    object_extensions.update({
        'course_id': current_event['context']['course_id'],
        'org_id': caliper_event['extensions']['extra_fields'].pop('org_id'),
    })

    caliper_object = {
        'id': object_link,
        'type': 'Group',
        'extensions': object_extensions
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
    })

    return caliper_event


def edx_team_searched(current_event, caliper_event):
    """
    When a user performs a search for teams from the topic view under the
    Teams page of the courseware, the server emits an edx.team.searched event.
    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_extensions = current_event['event']

    object_extensions.update({
        'course_id': current_event['context']['course_id'],
        'org_id': caliper_event['extensions']['extra_fields'].pop('org_id'),
    })

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Group',
        'extensions': object_extensions
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Searched',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
    })

    return caliper_event


def edx_team_learner_removed(current_event, caliper_event):
    """
    When a user leaves a team or is removed by someone else,
    the server emits an edx.team.learner_deleted event.

    This event is also triggered when a team is deleted,
    because all members are removed when a team is deleted.

    Course team members who have any of the Staff, Admin, Discussion Admin,
    Discussion Moderator, or Community TA roles can remove learners from teams.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    username = utils.get_username_from_user_id(current_event['event']['user_id'])

    user_link = utils.get_user_link_from_username(username)

    if current_event['event']['remove_method'] == 'team_deleted':
        object_link = current_event['referer']
    else:
        object_link = utils.get_team_url_from_team_id(
            current_event['referer'],
            current_event['event']['team_id']
        )

    caliper_object = {
        'id': object_link,
        'member': {
            'extensions': {
                'user_id': current_event['event']['user_id']
            },
            'id': user_link,
            'name': username,
            'type': 'Person'
        },
        'organization': {
            'extensions': {
                'team_id': current_event['event']['team_id']
            },
            'id': object_link,
            'type': 'Group'
        },
        'type': 'Membership',
        'extensions': {
            'remove_method': current_event['event']['remove_method']
        }
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Removed',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
    })

    return caliper_event


def edx_team_created(current_event, caliper_event):
    """
    This event is generated when new team is created.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    team_link = utils.get_team_url_from_team_id(
        current_event['referer'],
        current_event['event']['team_id']
    )

    caliper_object = {
        'extensions': current_event['event'],
        'id': team_link,
        'type': 'Group'
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Created',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
    })

    return caliper_event


def edx_team_activity_updated(current_event, caliper_event):
    """
    When team discussion activity has occurred on a team,
    including a team member posting, editing posts, commenting,
    responding, endorsing, and so on), the server emits
    an edx.team.activity_updated event.

    The definition of activity that would trigger
    this event does not include changes in team membership.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_link = utils.get_team_url_from_team_id(
        current_event['referer'],
        current_event['event']['team_id']
    )

    caliper_object = {
        'id': object_link,
        'type': 'Forum',
        'extensions': {
            'team_id': current_event['event']['team_id']
        }
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'course_user_tags': current_event['context']['course_user_tags'],
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
    })

    return caliper_event



"""
Apply required settings for caliperization of events.

Add CaliperProcessor to event tracking backends' list.
"""
from django.conf import settings as django_settings

from openedx_caliper_tracking import processor
from openedx_caliper_tracking.settings import OPENEDX_CALIPER_TRACKING_BACKENDS, OPENEDX_CALIPER_TRACKING_PROCESSOR

default_app_config = 'openedx_caliper_tracking.apps.CaliperTrackingConfig'

if hasattr(django_settings, 'EVENT_TRACKING_BACKENDS'):
    django_settings.EVENT_TRACKING_BACKENDS['tracking_logs']['OPTIONS']['processors'] += [
        {'ENGINE': OPENEDX_CALIPER_TRACKING_PROCESSOR}
    ]

if hasattr(django_settings, 'TRACKING_BACKENDS'):
    django_settings.TRACKING_BACKENDS = OPENEDX_CALIPER_TRACKING_BACKENDS




"""
Transformers for all the textbook interaction events
"""
import json


def textbook_pdf_page_scrolled(current_event, caliper_event):
    """
    The browser emits textbook.pdf.page.scrolled events each time the
    displayed page changes while a user scrolls up or down.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])
    current_event_details.pop('name')

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': current_event_details
    }
    caliper_event.update({
        'action': 'NavigatedTo',
        'type': 'NavigationEvent',
        'object': caliper_event_object
    })
    return caliper_event


def textbook_pdf_search_executed(current_event, caliper_event):
    """
    The browser emits textbook.pdf.search.executed events when
    a user searches for a text value in the file.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])
    current_event_details.pop('name')

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': current_event_details
    }

    caliper_event.update({
        'action': 'Searched',
        'type': 'Event',
        'object': caliper_event_object
    })

    return caliper_event


def textbook_pdf_page_navigated(current_event, caliper_event):
    """
    The browser emits textbook.pdf.page.navigated events when a
    user manually enters a page number.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': {
            'chapter': current_event_details['chapter'],
            'page': current_event_details['page']
        }
    }

    caliper_event.update({
        'action': 'NavigatedTo',
        'type': 'NavigationEvent',
        'object': caliper_event_object
    })

    return caliper_event


def textbook_pdf_zoom_menu_changed(current_event, caliper_event):
    """
    The browser emits textbook.pdf.zoom.menu.changed events when
    a user selects a magnification setting.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': current_event_details
    }
    caliper_event.update({
        'action': 'ChangedSize',
        'type': 'Event',
        'object': caliper_event_object
    })
    return caliper_event


def book(current_event, caliper_event):
    """
    The browser emits book events when a user navigates within the PDF
    Viewer or the PNG Viewer.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': current_event_details
    }

    caliper_event.update({
        'action': 'NavigatedTo',
        'type': 'NavigationEvent',
        'object': caliper_event_object
    })

    return caliper_event


def textbook_pdf_search_navigatednext(current_event, caliper_event):
    """
    The browser emits textbook.pdf.search.navigatednext
    events when a user clicks on the Find Next or Find Previous
    icons for an entered search string.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': json.loads(current_event['event'])
    }

    caliper_event.update({
        'action': 'NavigatedTo',
        'type': 'NavigationEvent',
        'object': caliper_event_object
    })

    return caliper_event


def textbook_pdf_thumbnail_navigated(current_event, caliper_event):
    """
    The browser emits textbook.pdf.thumbnail.navigated events when a user
    clicks on a thumbnail image to navigate to a page.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])
    current_event_details.pop('name')

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': current_event_details
    }
    caliper_event.update({
        'action': 'NavigatedTo',
        'type': 'NavigationEvent',
        'object': caliper_event_object
    })
    return caliper_event


def textbook_pdf_zoom_buttons_changed(current_event, caliper_event):
    """
    The browser emits textbook.pdf.zoom.buttons.changed events when
    a user clicks either the Zoom In or Zoom Out icon.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': current_event_details
    }
    caliper_event.update({
        'action': 'ChangedSize',
        'type': 'Event',
        'object': caliper_event_object
    })
    return caliper_event


def textbook_pdf_searchcasesensitivity_toggled(current_event, caliper_event):
    """
    The browser emits textbook.pdf.searchcasesensitivity.toggled events when a
    user selects or clears the Match Case option for a search.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'SoftwareApplication',
        'extensions': current_event_details
    }

    caliper_event.update({
        'action': 'Used',
        'type': 'ToolUseEvent',
        'object': caliper_event_object
    })

    return caliper_event


def textbook_pdf_search_highlight_toggled(current_event, caliper_event):
    """
    The browser emits textbook.pdf.search.highlight.toggled events when a user
    selects or clears the Highlight All option for a search.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'SoftwareApplication',
        'extensions': current_event_details
    }

    caliper_event.update({
        'action': 'Used',
        'type': 'ToolUseEvent',
        'object': caliper_event_object
    })

    return caliper_event


def textbook_pdf_thumbnails_toggled(current_event, caliper_event):
    """
    The browser emits textbook.pdf.thumbnails.toggled events when a user
    clicks on the icon to show or hide page thumbnails.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    current_event_details = json.loads(current_event['event'])
    current_event_details.pop('name')

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip'],
    })

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'SoftwareApplication',
        'extensions': current_event_details
    }

    caliper_event.update({
        'action': 'Used',
        'type': 'ToolUseEvent',
        'object': caliper_event_object
    })

    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def textbook_pdf_display_scaled(current_event, caliper_event):
    """
    The browser emits textbook.pdf.display.scaled events when the display
    magnification changes. These changes occur after a student selects a
    magnification setting from the zoom menu or resizes the browser window.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': current_event_details
    }
    caliper_event.update({
        'action': 'ChangedSize',
        'type': 'Event',
        'object': caliper_event_object
    })
    return caliper_event


def textbook_pdf_outline_toggled(current_event, caliper_event):
    """
    The browser emits textbook.pdf.outline.toggled events when a user clicks
    the outline icon to show or hide a list of the book's chapters.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    current_event_details = json.loads(current_event['event'])

    current_event_details.pop('name')

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip'],
    })

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'SoftwareApplication',
        'extensions': current_event_details
    }

    caliper_event.update({
        'action': 'Used',
        'type': 'ToolUseEvent',
        'object': caliper_event_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def textbook_pdf_chapter_navigated(current_event, caliper_event):
    """
    The browser emits textbook.pdf.page.scrolled events each time the
    displayed page changes while a user scrolls up or down.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event['ip'],
    })

    current_event_details = json.loads(current_event['event'])

    caliper_event_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': current_event_details
    }
    caliper_event.update({
        'action': 'NavigatedTo',
        'type': 'NavigationEvent',
        'object': caliper_event_object
    })
    return caliper_event



"""
Transformers for all the video events
"""

import json
from datetime import timedelta

from isodate import duration_isoformat


def pause_video(current_event, caliper_event):
    """
    When a user selects the video player's pause control,
    the player emits a pause_video event. For videos that
    are streamed in a browser, when the player reaches the
    end of the video file and play automatically stops it
    emits both this event and a stop event (as of June 2014).

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    current_event_details = json.loads(current_event['event'])
    caliper_event.update({
        'action': 'Paused',
        'type': 'MediaEvent',
        'object': {
            'duration': duration_isoformat(timedelta(
                seconds=current_event_details['duration']
            )),
            'extensions': {
                'code': current_event_details['code'],
                'id': current_event_details['id']
            },
            'id': current_event['referer'],
            'type': 'VideoObject'
        },
        'target': {
            'currentTime': duration_isoformat(timedelta(
                seconds=current_event_details['currentTime']
            )),
            'id': current_event['referer'],
            'type': 'MediaLocation'
        }
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def stop_video(current_event, caliper_event):
    """
    When the video player reaches the end of the video file and play
    automatically stops, the player emits a stop_video event.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    current_event_details = json.loads(current_event['event'])
    caliper_object = {
        'id': current_event['referer'],
        'type': 'VideoObject',
        'duration': duration_isoformat(
            timedelta(seconds=current_event_details['duration'])
        ),
        'extensions': {
            'code': current_event_details['code'],
            'id': current_event_details['id']
        }
    }
    caliper_event.update({
        'type': 'MediaEvent',
        'action': 'Ended',
        'object': caliper_object,
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })
    return caliper_event


def edx_video_speed_changed(current_event, caliper_event):
    """
    When the speed of the video is changed.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_event.update({
        'action': 'ChangedSpeed',
        'type': 'MediaEvent'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event.get('ip')
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['actor'].update({
        'name': current_event.get('username'),
        'type': 'Person'
    })
    event_info = json.loads(current_event['event'])
    caliper_event['object'] = {
        'id': current_event.get('referer'),
        'type': 'VideoObject',
        'duration': duration_isoformat(timedelta(
            seconds=event_info.pop('duration')
        )),
    }
    caliper_event['target'] = {
        'id': current_event['referer'],
        'type': 'MediaLocation',
        'currentTime': duration_isoformat(timedelta(
            seconds=event_info.pop('current_time')
        ))
    }
    caliper_event['object']['extensions'] = event_info
    return caliper_event


def play_video(current_event, caliper_event):
    """
    When a user selects the video player's play control,
    the player emits a play_video event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """

    event_info = json.loads(current_event['event'])

    caliper_event.update({
        'action': 'Started',
        'type': 'MediaEvent',
        'target': {
            'currentTime': duration_isoformat(timedelta(
                seconds=event_info['currentTime']
            )),
            'id': current_event['referer'],
            'type': 'MediaLocation'
        }
    })

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event.get('ip')
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event.get('username'),
        'type': 'Person'
    })

    caliper_event['object'] = {
        'id': current_event.get('referer'),
        'type': 'VideoObject',
        'duration': duration_isoformat(timedelta(
            seconds=event_info.pop('duration')
        )),
        'extensions': {
            'id': event_info.get('id'),
            'code': event_info.get('code'),
        }
    }
    return caliper_event


def load_video(current_event, caliper_event):
    """
    When the video is fully rendered and ready to play,
    the browser or mobile app emits a load_video event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_event.update({
        'action': 'Retrieved',
        'type': 'Event'
    })

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context'].get('course_id'),
        'ip': current_event.get('ip')
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event.get('username'),
        'type': 'Person'
    })

    event_info = json.loads(current_event['event'])

    caliper_event['object'] = {
        'id': current_event.get('referer'),
        'type': 'VideoObject',
        'duration': duration_isoformat(timedelta(
            seconds=event_info.pop('duration')
        )),
        'extensions': {
            'id': event_info.get('id'),
            'code': event_info.get('code'),
        }
    }

    return caliper_event


def seek_video(current_event, caliper_event):
    """
    A browser emits seek_video events when a user selects a user interface control to go to a different point in the
    video file.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    current_event_details = json.loads(current_event['event'])
    caliper_event.update({
        'action': 'JumpedTo',
        'type': 'MediaEvent',
        'object': {
            'duration': duration_isoformat(
                timedelta(seconds=current_event_details['duration'])),
            'extensions': {
                'code': current_event_details['code'],
                'id': current_event_details['id'],
                'type': current_event_details['type'],
            },
            'id': current_event['referer'],
            'type': 'VideoObject'
        },
        'target': {
            'id': current_event['referer'],
            'type': 'MediaLocation',
            'currentTime': duration_isoformat(
                timedelta(seconds=current_event_details['new_time'])),
            'extensions': {
                'old_time': duration_isoformat(
                    timedelta(seconds=current_event_details['old_time'])),
            }
        }
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_video_closed_captions_shown(current_event, caliper_event):
    """
    When a user toggles CC to display the closed captions, the browser or mobile app emits an
    edx.video.closed_captions.shown event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    current_event_details = json.loads(current_event['event'])
    caliper_event.update({
        'action': 'EnabledClosedCaptioning',
        'type': 'MediaEvent',
        'object': {
            'duration': duration_isoformat(
                timedelta(seconds=current_event_details['duration'])),
            'extensions': {
                'code': current_event_details['code'],
                'id': current_event_details['id']
            },
            'id': current_event['referer'],
            'type': 'VideoObject'
        },
        'target': {
            'currentTime': duration_isoformat(
                timedelta(seconds=current_event_details['current_time'])),
            'id': current_event['referer'],
            'type': 'MediaLocation'
        }
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def edx_video_closed_captions_hidden(current_event, caliper_event):
    """
    When a user toggles CC to display the closed captions, the browser or mobile app emits an
    edx.video.closed_captions.shown event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    current_event_details = json.loads(current_event['event'])
    caliper_event.update({
        'action': 'DisabledClosedCaptioning',
        'type': 'MediaEvent',
        'object': {
            'duration': duration_isoformat(
                timedelta(seconds=current_event_details['duration'])),
            'extensions': {
                'code': current_event_details['code'],
                'id': current_event_details['id']
            },
            'id': current_event['referer'],
            'type': 'VideoObject'
        },
        'target': {
            'currentTime': duration_isoformat(
                timedelta(seconds=current_event_details['current_time'])),
            'id': current_event['referer'],
            'type': 'MediaLocation'
        }
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def hide_transcript(current_event, caliper_event):
    """
    When a user toggles Show Transcript to suppress display of the video transcript, the browser or mobile app emits a
    hide_transcript event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    current_event_details = json.loads(current_event['event'])
    caliper_event.update({
        'action': 'DisabledClosedCaptioning',
        'type': 'MediaEvent',
        'object': {
            'duration': duration_isoformat(
                timedelta(seconds=current_event_details['duration'])),
            'extensions': {
                'code': current_event_details['code'],
                'id': current_event_details['id']
            },
            'id': current_event['referer'],
            'type': 'VideoObject'
        },
        'target': {
            'currentTime': duration_isoformat(
                timedelta(seconds=current_event_details['current_time'])),
            'id': current_event['referer'],
            'type': 'MediaLocation'
        }
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def show_transcript(current_event, caliper_event):
    """
    When a user toggles Show Transcript to display the video transcript, the browser or mobile app emits a
    show_transcript event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    current_event_details = json.loads(current_event['event'])
    caliper_event.update({
        'action': 'EnabledClosedCaptioning',
        'type': 'MediaEvent',
        'object': {
            'duration': duration_isoformat(
                timedelta(seconds=current_event_details['duration'])),
            'extensions': {
                'code': current_event_details['code'],
                'id': current_event_details['id']
            },
            'id': current_event['referer'],
            'type': 'VideoObject'
        },
        'target': {
            'currentTime': duration_isoformat(
                timedelta(seconds=current_event_details['current_time'])),
            'id': current_event['referer'],
            'type': 'MediaLocation'
        }
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def video_hide_cc_menu(current_event, caliper_event):
    """
    When a user closes the Language Menu for a video that has transcripts in multiple languages, the browser emits a
    video_hide_cc_menu event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    current_event_details = json.loads(current_event['event'])
    caliper_event.update({
        'action': 'Hid',
        'type': 'Event',
        'object': {
            'duration': duration_isoformat(
                timedelta(seconds=current_event_details['duration'])),
            'extensions': {
                'code': current_event_details['code'],
                'id': current_event_details['id']
            },
            'id': current_event['referer'],
            'type': 'Frame'
        }
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip'],
        'name': current_event['name']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def video_show_cc_menu(current_event, caliper_event):
    """
    When a user opens the Language Menu for a video that has transcripts in multiple languages,
    the browser emits a video_show_cc_menu event.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    current_event_details = json.loads(current_event['event'])
    caliper_event.update({
        'action': 'Showed',
        'type': 'Event',
        'object': {
            'duration': duration_isoformat(
                timedelta(seconds=current_event_details['duration'])),
            'extensions': {
                'code': current_event_details['code'],
                'id': current_event_details['id']
            },
            'id': current_event['referer'],
            'type': 'Frame'
        }
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip'],
        'name': current_event['name']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event



import logging
import json
import requests

from django.conf import settings
from requests.exceptions import ConnectionError
from track.backends import BaseBackend

from openedx_caliper_tracking.base_transformer import base_transformer, page_view_transformer
from openedx_caliper_tracking.caliper_config import EVENT_MAPPING
from openedx_caliper_tracking.loggers import get_caliper_logger

LOGGER = logging.getLogger(__name__)
TRACKING_LOGGER = logging.getLogger('tracking')
CALIPER_LOGGER = get_caliper_logger('caliper')


def log_success(event_id, status_code):
    """
    This function logs the successful delivery of the caliper event
    to the external API

    @params:
    event_id: (str) UUID of the caliper event
    status_code: (int) HTTP status code of the repsonse from the API
    """
    LOGGER.info('Success {}: Caliper event delivery successful for event id: {} to endpoint: {}'.format(
        status_code,
        event_id,
        settings.CALIPER_DELIVERY_ENDPOINT
    ))


def log_failure(event_id, status_code):
    """
    This function logs the failed delivery of the caliper event
    to the external API

    @params:
    event_id: (str) UUID of the caliper event
    status_code: (int) HTTP status code of the repsonse from the API
    """
    LOGGER.error('Failure {}: Caliper event delivery failed for event id: {} to endpoint: {}'.format(
        status_code,
        event_id,
        settings.CALIPER_DELIVERY_ENDPOINT
    ))

def deliver_caliper_event(caliperized_event, event_type):
    """
    Delivers the caliperized event to the external API endpoint.

    @params
    caliperized_event: (dict) dict containing the entire event after caliperization
    event_type: (str) the type of the event being fired
    """
    try:
        response = requests.post(
            settings.CALIPER_DELIVERY_ENDPOINT,
            headers={
                'Authorization': 'Bearer {}'.format(
                    settings.CALIPER_DELIVERY_AUTH_TOKEN),
                'Content-Type': 'application/vnd.kafka.json.v2+json',
            },
            json={
                "records": [
                    {
                        "key": event_type,
                        "value": caliperized_event
                    }
                ]
            }
        )

        if response.status_code == 200:
            log_success(caliperized_event.get('id'), response.status_code)
        else:
            log_failure(caliperized_event.get('id'), response.status_code)
    except ConnectionError:
        log_failure(caliperized_event.get('id'), 500)


class CaliperProcessor(BaseBackend):
    """
    Is responsible for capturing, transforming and sending all the event tracking logs
    generated by Open edX platform.

    This transformer is used in the commong.djangoapps.track django app as a replacement
    for the default tracking backend.

    It is also used as an addition to the event tracking pipeline in the event_tracking app
    by Open edX
    """

    def __call__(self, event):
        """
        handles the transformation and delivery of an event. Delivers the event to the caliper log file as well as
        delivers it to an external API if configured to do so.

        @params:
        event: raw event from edX event tracking pipeline
        """
        try:
            caliper_event = base_transformer(event)
            related_function = EVENT_MAPPING[event.get('event_type')]
            transformed_event = related_function(event, caliper_event)

            CALIPER_LOGGER.info(json.dumps(transformed_event))

            if getattr(settings, 'CALIPER_DELIVERY_ENDPOINT') and getattr(settings, 'CALIPER_DELIVERY_AUTH_TOKEN'):
                deliver_caliper_event(transformed_event, event.get('event_type'))

            return event
        except KeyError:
            TRACKING_LOGGER.exception("Missing transformer method implementation for {}".format(
                event.get('event_type')))
        except Exception as ex:
            TRACKING_LOGGER.exception(ex.args)

    def send(self, event):
        """
        Implements the abstract send method in track.backends.BaseBackend

        @params:
        event: (dict) raw event from edX event tracking pipeline:
        """
        if not event['event_type'].startswith('/'):
            TRACKING_LOGGER.info(self.__call__(event))
        else:
            TRACKING_LOGGER.info(json.dumps(event))
            CALIPER_LOGGER.info(page_view_transformer(event))



"""
Transformers for all exam related events.
"""
from openedx_caliper_tracking.utils import convert_datetime


def edx_special_exam_timed_attempt_created(current_event, caliper_event):
    """
    The server emits this event when a learner chooses to take a special exam.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Created',
        'object': {
            'id': current_event['referer'],
            'type': 'Attempt',
            'name': current_event['event'].pop('exam_name'),
            'extensions': current_event['event']
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def edx_special_exam_timed_attempt_submitted(current_event, caliper_event):
    """
    The server emits this event when a learner completes a proctored exam
    and submits it for grading and review.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'AssessmentEvent',
        'action': 'Submitted',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
            'name': current_event['event'].pop('exam_name'),
            'extensions': current_event['event']
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context'].get('course_user_tags'),
        'ip': current_event['ip']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def edx_special_exam_timed_attempt_started(current_event, caliper_event):
    """
    The server emits this event when a learner begins taking a proctored exam.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'AssessmentEvent',
        'action': 'Started',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
            'name': current_event['event'].pop('exam_name'),
            'extensions': current_event['event']
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def edx_special_exam_timed_attempt_deleted(current_event, caliper_event):
    """
    The server emits this event when a course team or edX platform administrator
    removes an exam attempt record for an individual learner.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Deleted',
        'object': {
            'id': current_event['referer'],
            'type': 'Attempt',
        }
    })
    if current_event['event'].get('exam_name'):
        caliper_event['object']['name'] = current_event['event'].pop('exam_name')

    if current_event['event'].get('attempt_started_at'):
        caliper_event['object']['startedAtTime'] = convert_datetime(
            current_event['event'].pop(
                'attempt_started_at'
            ))

    if current_event['event'].get('attempt_completed_at'):
        caliper_event['object']['endedAtTime'] = convert_datetime(
            current_event['event'].pop(
                'attempt_completed_at'
            ))

    caliper_event['object'].update({
        'extensions': current_event['event']
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def edx_special_exam_practice_attempt_created(current_event, caliper_event):
    """
    The server emits this event when a learner chooses to take a special exam.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Created',
        'object': {
            'id': current_event['referer'],
            'type': 'Attempt',
            'name': current_event['event'].pop('exam_name'),
            'extensions': current_event['event']
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event


def edx_special_exam_proctored_created(current_event, caliper_event):
    """
    The server emits this event when a course instructor creates Proctored Exam.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Created',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
        }
    })

    if current_event['event'].get('exam_name'):
        caliper_event['object']['name'] = current_event['event'].pop('exam_name')

    caliper_event['object'].update({
        'extensions': current_event['event']
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_special_exam_practice_created(current_event, caliper_event):
    """
    The server emits this event when a course instructor creates Practice Exam.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Created',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
        }
    })

    if current_event['event'].get('exam_name'):
        caliper_event['object']['name'] = current_event['event'].pop('exam_name')

    caliper_event['object'].update({
        'extensions': current_event['event']
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_special_exam_practice_updated(current_event, caliper_event):
    """
    The server emits this event when a instructor changes exam type.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
        }
    })

    if current_event['event'].get('exam_name'):
        caliper_event['object']['name'] = current_event['event'].pop('exam_name')

    caliper_event['object'].update({
        'extensions': current_event['event']
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_special_exam_timed_updated(current_event, caliper_event):
    """
    The server emits this event when a instructor changes the time limit for timed exams.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
        }
    })

    if current_event['event'].get('exam_name'):
        caliper_event['object']['name'] = current_event['event'].pop('exam_name')

    caliper_event['object'].update({
        'extensions': current_event['event']
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_special_exam_proctored_updated(current_event, caliper_event):
    """
    The server emits this event when a instructor changes exam type.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
        }
    })

    if current_event['event'].get('exam_name'):
        caliper_event['object']['name'] = current_event['event'].pop('exam_name')

    caliper_event['object'].update({
        'extensions': current_event['event']
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_special_exam_timed_created(current_event, caliper_event):
    """
    The server emits this event when a course instructor creates Timed Exam.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'Event',
        'action': 'Created',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
        }
    })

    if current_event['event'].get('exam_name'):
        caliper_event['object']['name'] = current_event['event'].pop('exam_name')

    caliper_event['object'].update({
        'extensions': current_event['event']
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_special_exam_time_attempt_ready_to_submit(current_event, caliper_event):
    """
    The server emits this event when a learner has completed a proctored exam
    and is prompted to submit the exam.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': {
            'id': current_event['referer'],
            'type': 'WebPage',
            'name': current_event['event'].pop('exam_name'),
            'extensions': current_event['event']
        }
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })
    caliper_event['referrer']['type'] = 'WebPage'
    return caliper_event



"""
Transformers for all the navigation events
"""
import json


def edx_course_student_notes_edited(current_event, caliper_event):
    """
    The browser emits edx.course.student_notes.edited events when a
    learner edits a note in course.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_extensions = json.loads(current_event['event'])

    object_extensions.update({
        'course_id': current_event['context']['course_id'],
        'org_id': current_event['context']['org_id'],
    })

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': object_extensions
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Modified',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
    })

    return caliper_event


def edx_course_student_notes_added(current_event, caliper_event):
    """
    The server emits an edx.course.student_notes.added event when the student
    add notes on a textual content present in course.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_event.update(
        {
            'type': 'AnnotationEvent',
            'action': 'Tagged',
            'object': {
                'id': current_event['referer'],
                'type': 'DigitalResource',
                'extensions': json.loads(current_event['event'])
            }
        }
    )
    caliper_event['object']['extensions']['course_id'] = \
        current_event['context']['course_id']
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'path': current_event['context']['path'],
    })
    return caliper_event


def edx_course_student_notes_viewed(current_event, caliper_event):
    """
    The server emits an edx.course.student_notes.viewed event when the student
    views notes on a textual content present in course.

    :param current_event: default logs generated by edx itself
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': json.loads(current_event['event'])
    }

    caliper_object['extensions'].update({
        'course_id': current_event['context']['course_id'],
        'org_id': caliper_event['extensions']['extra_fields'].pop('org_id')
    })

    caliper_event.update({
        'type': 'ViewEvent',
        'action': 'Viewed',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'event_source': current_event['event_source'],
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_course_student_notes_deleted(current_event, caliper_event):
    """
    The browser emits edx.course.student_notes.deleted events when a
    learner deletes a note in course.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    object_extensions = json.loads(current_event['event'])

    object_extensions.update({
        'course_id': current_event['context']['course_id'],
        'org_id': caliper_event['extensions']['extra_fields'].pop('org_id'),
    })

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': object_extensions
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Deleted',
        'object': caliper_object
    })

    caliper_event['referrer']['type'] = 'WebPage'

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
    })

    return caliper_event


def edx_course_student_notes_searched(current_event, caliper_event):
    """
    The server emits an edx.course.student_notes.searched event when the student
    searches notes on a textual content present in course.

    :param current_event: default logs generated by edx itself
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'Document',
        'extensions': json.loads(current_event['event'])
    }

    caliper_object['extensions'].update({
        'course_id': current_event['context']['course_id'],
        'org_id': caliper_event['extensions']['extra_fields'].pop('org_id'),
        'user_id': caliper_event['extensions']['extra_fields'].pop('user_id')
    })

    caliper_event.update({
        'type': 'Event',
        'action': 'Searched',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'event_source': current_event['event_source'],
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_course_student_notes_used_unit_link(current_event, caliper_event):
    """
    This event is generated when the user clicks on the note link.

    :param current_event: default logs generated by edx itself
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'WebPage',
        'extensions': json.loads(current_event['event'])
    }

    caliper_object['extensions'].update({
        'course_id': current_event['context']['course_id'],
        'org_id': caliper_event['extensions']['extra_fields'].pop('org_id')
    })

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'event_source': current_event['event_source']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event



"""
Transformers for all the problems events
"""
import json
import logging
from django.conf import settings


logger = logging.getLogger(__name__)


def problem_reset(current_event, caliper_event):
    """
    The browser emits problem_reset events when the answer to a
    problem given before is reset; that is, the user selected
    Reset.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_object = {
        'id': current_event['page'],
        'type': 'Assessment'
    }
    caliper_event.update({
        'action': 'Reset',
        'type': 'AssessmentEvent',
        'object': caliper_object
    })
    caliper_event['referrer'].update({
        'type': 'WebPage'
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'event': current_event['event'],
        'ip': current_event['ip']
    })
    return caliper_event


def problem_show(current_event, caliper_event):
    """
    The browser emits problem_show events when the answer to a
    problem is shown; that is, the user selected Show Answer.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    current_event_details = json.loads(current_event['event'])

    caliper_object = {
        'id': current_event['referer'],
        'extensions': {
            'problem': current_event_details['problem']
        },
        'type': 'DigitalResource'
    }
    caliper_event.update({
        'action': 'Viewed',
        'type': 'ViewEvent',
        'object': caliper_object
    })
    caliper_event['referrer'].update({
        'type': 'WebPage'
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'event': current_event['event'],
        'ip': current_event['ip'],
    })
    return caliper_event


def problem_graded(current_event, caliper_event):
    """
    The browser emits problem_graded events when the answer to a
    problem is submitted; that is, the user selected Submitted.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'extensions': {
            'event': current_event['event']
        },
        'type': 'Attempt'
    }
    caliper_event.update({
        'action': 'Graded',
        'type': 'GradeEvent',
        'object': caliper_object
    })
    caliper_event['referrer'].update({
        'type': 'WebPage'
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip'],
    })
    return caliper_event


def problem_save(current_event, caliper_event):
    """
    The browser emits problem_save events when the answer to a
    problem is save instead of submitting; that is,
    the user selected Save Answer.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'extensions': {
            'event': current_event['event']
        },
        'type': 'Assessment'
    }
    caliper_event.update({
        'action': 'Paused',
        'type': 'AssessmentEvent',
        'object': caliper_object
    })
    caliper_event['referrer'].update({
        'type': 'WebPage'
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip']
    })
    return caliper_event


def save_problem_success(current_event, caliper_event):
    """
    The server emits save_problem_success events when a problem is saved successfully.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event.update({
        'action': 'Paused',
        'type': 'AssessmentEvent',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
            'extensions': current_event['event']
        }
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'asides': current_event['context']['asides'],
        'course_user_tags': current_event['context']['course_user_tags'],
        'module': current_event['context']['module'],
        'ip': current_event['ip']
    })
    return caliper_event


def problem_check(current_event, caliper_event):
    """
    The server emits problem_check events when a problem is successfully checked.
    Both browser interactions and server requests produce problem_check events, so your data package
    can also contain events with an event source of browser.
    Events emitted by the browser contain all of the GET parameters.
    Only events emitted by the server are useful for most purposes.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_event.update({
        'action': 'Submitted',
        'type': 'AssessmentEvent',
        'object': {
            'id': current_event['referer'],
            'type': 'Assessment',
        }
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']

    if current_event.get('event_source') == 'server':
        caliper_event['extensions']['extra_fields'].pop('session')
        caliper_event['object']['extensions'] = current_event['event']
    else:
        caliper_event['object'].update({
            'extensions': {
                'event': current_event['event']
            }
        })

    return caliper_event


def edx_problem_hint_demandhint_displayed(current_event, caliper_event):
    """
    Course teams can design problems to include one or more hints. For problems that include hints,
    the server emits an edx.problem.hint.demandhint_displayed event each time a user requests a
    hint.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'extensions': current_event['event'],
        'type': 'Frame'
    }

    caliper_event.update({
        'action': 'Viewed',
        'type': 'ViewEvent',
        'object': caliper_object
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']

    return caliper_event


def problem_rescore(current_event, caliper_event):
    """
    The server emits problem_rescore events when a problem is successfully
    rescored.
    In these events, the user who rescored the problem is identified in the
    username and context.user_id fields, and the user who originally submitted
    the response to the problem is identified in the student field.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    if not (current_event.get('referer') or current_event['context'].get('referer')):
        logger.exception(
            'Missing "referer" key in event which is required for "object.id"'
            'in caliper events, therefore returning the original event'
        )
        current_event['id'] = None
        return current_event

    caliper_object = {
        'id': current_event['context'].get('referer'),
        'extensions': current_event['event'],
        'type': 'Attempt'
    }

    caliper_event.update({
        'action': 'Graded',
        'type': 'GradeEvent',
        'object': caliper_object
    })

    caliper_event['referrer'].update({
        'id': current_event['context'].pop('referer'),
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['context'].pop('username'),
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    return caliper_event


def edx_problem_hint_feedback_displayed(current_event, caliper_event):
    """
    Course teams can design problems to include feedback messages that appear
    after a user submits an answer.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'extensions': current_event['event'],
        'type': 'Frame'
    }

    caliper_event.update({
        'action': 'Viewed',
        'type': 'ViewEvent',
        'object': caliper_object
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields'].pop('session')
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']

    return caliper_event


def edx_grades_problem_submitted(current_event, caliper_event):
    """
    Occurs when learner submitted submitted a question of peer assignment.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'extensions': current_event['event'],
        'type': 'Assessment'
    }

    caliper_event.update({
        'action': 'Submitted',
        'type': 'AssessmentEvent',
        'object': caliper_object
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']

    return caliper_event


def edx_grades_problem_state_deleted(current_event, caliper_event):
    """
    When a course team member deletes the state for a learner's problem
    submission, the server emits an edx.grades.problem.state_deleted event.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'extensions': current_event['event'],
        'type': 'Assessment'
    }

    caliper_event.update({
        'action': 'Reset',
        'type': 'AssessmentEvent',
        'object': caliper_object
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']

    return caliper_event


def edx_grades_problem_rescored(current_event, caliper_event):
    """
    Server emits this event when problem re-scoring task is accomplished.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    if not current_event.get('referer'):
        logger.exception(
            'Missing "referer" key in event which is required for "object.id"'
            'in caliper events, therefore returning the original event'
        )
        current_event['id'] = None
        return current_event

    caliper_object = {
        'id': current_event['referer'],
        'extensions': current_event['event'],
        'type': 'Attempt'
    }

    caliper_event.update({
        'action': 'Graded',
        'type': 'GradeEvent',
        'object': caliper_object
    })

    caliper_event['referrer'].update({
        'id': current_event['referer'],
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']

    return caliper_event


def reset_problem_fail(current_event, caliper_event):
    """
    The server emits reset_problem_fail events when a problem cannot be reset successfully.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Assessment',
        'extensions': current_event['event']
    }
    caliper_event.update({
        'action': 'Reset',
        'type': 'AssessmentEvent',
        'object': caliper_object
    })
    caliper_event['referrer'].update({
        'type': 'WebPage'
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],

    })
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event


def edx_grades_problem_score_overridden(current_event, caliper_event):
    """
    Server emits this event when a user's score for a problem is overridden.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    if not current_event.get('referer'):
        logger.exception(
            'Missing "referer" key in event which is required for "object.id"'
            'in caliper events, therefore returning the original event'
        )
        current_event['id'] = None
        return current_event

    caliper_object = {
        'id': current_event['referer'],
        'extensions': current_event['event'],
        'type': 'Attempt'
    }

    caliper_event.update({
        'action': 'Graded',
        'type': 'GradeEvent',
        'object': caliper_object
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'event_source': current_event['event_source'],
        'host': current_event['host'],
        'session': current_event['session'],
        'page': current_event['page'],
    })
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']

    return caliper_event


def reset_problem(current_event, caliper_event):
    """
    The server emits reset_problem events when a problem has been reset
    successfully.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_object = {
        'id': current_event['referer'],
        'type': 'Assessment',
        'extensions': current_event['event']
    }
    caliper_event.update({
        'action': 'Reset',
        'type': 'AssessmentEvent',
        'object': caliper_object
    })
    caliper_event['referrer'].update({
        'type': 'WebPage'
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields'].update({
        'event_source': current_event['event_source'],
        'event_type': current_event['event_type'],
        'host': current_event['host'],
        'ip': current_event['ip'],
        'page': current_event['page'],
    })
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event


def showanswer(current_event, caliper_event):
    """
    The server emits showanswer events when the answer to a problem is shown.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    caliper_object = {
        'id': current_event['referer'],
        'extensions': current_event['event'],
        'type': 'Frame'
    }
    caliper_event.update({
        'action': 'Viewed',
        'type': 'ViewEvent',
        'object': caliper_object
    })
    caliper_event['referrer'].update({
        'type': 'WebPage'
    })
    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })
    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    caliper_event['extensions']['extra_fields'].pop('session')

    return caliper_event


def problem_check_fail(current_event, caliper_event):
    """
    The server emits problem_check_fail events when a problem cannot be
    checked successfully.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    # transformer code for this event is same as that of problem_check
    return problem_check(current_event, caliper_event)


def save_problem_fail(current_event, caliper_event):
    """
    The server emits save_problem_fail events when a problem cannot be
    saved successfully.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_event.update({
        'action': 'Abandoned',
        'type': 'Event',
        'object': {
            'id': current_event['referer'],
            'type': 'AssessmentItem',
            'extensions': current_event['event']
        }
    })
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['actor'].update({
        'id': settings.LMS_ROOT_URL,
        'type': 'SoftwareApplication',
    })
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']
    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields'].pop('session')
    return caliper_event



"""
Customized loggers for openedx-caliper-tracking app
"""

import logging
import sys

from django.conf import settings


def get_formatted_log(result, filename, event_name, status_code):
    """Return a formatted log message"""
    return '{} [{}][event_type:{}] Returned status: {}'.format(
        result, filename, event_name, status_code
    )


def get_test_logger(logger_name, file_path):
    """
    logger for caliper app tests
    :param logger_name: The name you want you create logger of
    :param file_path: The path where logs will be stored
    :return: Logger with defined configurations
    """

    logger = logging.getLogger(logger_name)

    f_handler = logging.FileHandler(file_path)
    f_handler.setLevel(logging.INFO)
    f_format = logging.Formatter(
        '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    f_handler.setFormatter(f_format)

    logger.addHandler(f_handler)
    logger.setLevel(logging.INFO)

    return logger


def get_caliper_logger(logger_name):
    """
    logger for caliper app

    :param logger_name: The name you want you create logger of
    :return: Logger with defined configurations
    """

    logger = logging.getLogger(logger_name)

    # checks if the environment is either devstack or test
    if settings.DEBUG or hasattr(settings, 'TEST_ROOT'):
        log_handler = logging.StreamHandler()
    else:
        log_handler = logging.handlers.SysLogHandler(address='/dev/log', facility='local2')

    log_handler.setLevel(logging.INFO)

    logger.addHandler(log_handler)
    logger.setLevel(logging.INFO)
    logger.propagate = False

    return logger



"""
Transformers for all the navigation events
"""
import json


def edx_ui_lms_link_clicked(current_event, caliper_event):
    """
    This trannsformer uses current_event to generate caliper_event
    with currently available data.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """

    current_event_details = json.loads(current_event['event'])
    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': {
            'id': current_event_details['target_url'],
            'type': 'WebPage'
        }
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['actor'].update({
        'name': current_event['username'],
        'type': 'Person'
    })

    caliper_event['extensions']['extra_fields'].update({
        'course_id': current_event['context']['course_id'],
        'ip': current_event['ip'],
    })

    return caliper_event


def edx_course_tool_accessed(current_event, caliper_event):
    """
    The browser emits an edx.course.tool.accessed event when a user clicks
    one of the links under the Course Tools heading in the LMS, such
    as Bookmarks, Reviews, or Updates.

    :param current_event: default log
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['page'],
        'type': 'WebPage',
    }

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update({
        'event': current_event['event'],
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    return caliper_event


def edx_ui_lms_sequence_previous_selected(current_event, caliper_event):
    """
    This event is generated when the user presses the previous button to
    traverse over unit of the previous chapter.

    :param current_event: default logs generated by edx itself
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'WebPage',
        'extensions': json.loads(current_event['event'])
    }

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_ui_lms_sequence_next_selected(current_event, caliper_event):
    """
    This event is generated when the user presses the next button to traverse
    over unit of next chapter.

    :param current_event: default logs generated by edx itself
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'WebPage',
        'extensions': json.loads(current_event['event'])
    }

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def seq_next(current_event, caliper_event):
    """
    This event is generated when the user presses the next button to traverse
    over next unit of same chapter.

    :param current_event: default logs generated by edx itself
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'WebPage',
        'extensions': json.loads(current_event['event'])
    }

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'event_source': current_event['event_source'],
        'course_id': current_event['context']['course_id'],
        'name': current_event['name']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def seq_prev(current_event, caliper_event):
    """
    This event is generated when the user presses the previous button to
    traverse over previous unit of same chapter.

    :param current_event: default logs generated by edx itself
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'WebPage',
        'extensions': json.loads(current_event['event'])
    }

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'event_source': current_event['event_source'],
        'course_id': current_event['context']['course_id'],
        'name': current_event['name']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def edx_course_student_notes_notes_page_viewed(current_event, caliper_event):
    """
    This event is generated when the user presses the notes button to see
    notes made on that course.

    :param current_event: default logs generated by edx itself
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'WebPage',
        'extensions': json.loads(current_event['event'])
    }

    caliper_object['extensions'].update({
        'course_id': current_event['context']['course_id'],
        'org_id': caliper_event['extensions']['extra_fields'].pop('org_id')
    })

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'event_source': current_event['event_source']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def seq_goto(current_event, caliper_event):
    """
    This event is generated when the user presses the previous button to traverse
    over previous unit of same chapter.

    :param current_event: default logs generated by edx itself
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'Page',
        'extensions': json.loads(current_event['event'])
    }

    caliper_event.update({
        'type': 'NavigationEvent',
        'action': 'NavigatedTo',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'name': current_event['name']
    })

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event


def page_close(current_event, caliper_event):
    """
    The page_close event originates from within the JavaScript Logger itself when
    a page is closed.

    :param current_event: default logs generated by edx itself
    :param caliper_event: log containing both basic and default attribute
    :return: final created log
    """
    caliper_object = {
        'id': current_event['referer'],
        'type': 'WebPage',
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Removed',
        'object': caliper_object,
    })

    caliper_event['extensions']['extra_fields'].update(current_event['context'])
    caliper_event['extensions']['extra_fields']['ip'] = current_event['ip']

    caliper_event['actor'].update({
        'type': 'Person',
        'name': current_event['username']
    })

    caliper_event['referrer']['type'] = 'WebPage'

    return caliper_event



"""
Exposes transformers functions.
"""
from openedx_caliper_tracking.transformers.bookmark_transformers import (
    edx_bookmark_listed,
    edx_bookmark_added,
    edx_bookmark_removed,
    edx_bookmark_accessed,
)
from openedx_caliper_tracking.transformers.navigation_transformers import (
    edx_ui_lms_link_clicked,
    edx_course_tool_accessed,
    edx_ui_lms_sequence_next_selected,
    edx_ui_lms_sequence_previous_selected,
    seq_prev,
    seq_next,
    edx_course_student_notes_notes_page_viewed,
    seq_goto,
    page_close,
)
from openedx_caliper_tracking.transformers.enrollment_transformers import (
    edx_course_enrollment_activated,
    edx_course_enrollment_mode_changed,
    edx_course_enrollment_deactivated,
    edx_course_enrollment_upgrade_clicked,
    edx_course_enrollment_upgrade_succeeded,
)
from openedx_caliper_tracking.transformers.problem_transformers import (
    problem_show,
    problem_save,
    problem_reset,
    save_problem_success,
    problem_check,
    edx_problem_hint_demandhint_displayed,
    problem_rescore,
    problem_graded,
    edx_problem_hint_feedback_displayed,
    edx_grades_problem_submitted,
    edx_grades_problem_state_deleted,
    edx_grades_problem_rescored,
    reset_problem_fail,
    edx_grades_problem_score_overridden,
    reset_problem,
    showanswer,
    problem_check_fail,
    save_problem_fail,
)
from openedx_caliper_tracking.transformers.video_transformers import (
    pause_video,
    stop_video,
    edx_video_speed_changed,
    play_video,
    load_video,
    seek_video,
    edx_video_closed_captions_shown,
    edx_video_closed_captions_hidden,
    show_transcript,
    hide_transcript,
    video_hide_cc_menu,
    video_show_cc_menu,
)
from openedx_caliper_tracking.transformers.forum_transformers import (
    edx_forum_response_created,
    edx_forum_thread_created,
    edx_forum_thread_viewed,
    edx_forum_comment_created,
    edx_forum_thread_voted,
    edx_forum_searched,
    edx_forum_response_voted,
)
from openedx_caliper_tracking.transformers.xblock_transformers import (
    xblock_poll_submitted,
    xblock_poll_view_results,
    xblock_survey_submitted,
    xblock_survey_view_results,
    xblock_split_test_child_render,
)
from openedx_caliper_tracking.transformers.textbook_transformers import (
    textbook_pdf_page_scrolled,
    textbook_pdf_search_executed,
    textbook_pdf_zoom_menu_changed,
    textbook_pdf_thumbnail_navigated,
    textbook_pdf_page_navigated,
    book,
    textbook_pdf_zoom_buttons_changed,
    textbook_pdf_outline_toggled,
    textbook_pdf_thumbnails_toggled,
    textbook_pdf_searchcasesensitivity_toggled,
    textbook_pdf_search_highlight_toggled,
    textbook_pdf_display_scaled,
    textbook_pdf_chapter_navigated,
    textbook_pdf_search_navigatednext,
)
from openedx_caliper_tracking.transformers.notes_transformers import (
    edx_course_student_notes_added,
    edx_course_student_notes_added,
    edx_course_student_notes_viewed,
    edx_course_student_notes_edited,
    edx_course_student_notes_deleted,
    edx_course_student_notes_searched,
    edx_course_student_notes_added,
    edx_course_student_notes_used_unit_link,
)
from openedx_caliper_tracking.transformers.open_response_transformers import (
    openassessmentblock_submit_feedback_on_assessments,
    openassessmentblock_save_submission,
    openassessmentblock_get_peer_submission,
    openassessmentblock_get_submission_for_staff_grading,
    openassessmentblock_peer_assess,
    openassessmentblock_create_submission,
    openassessment_student_training_assess_example,
    openassessmentblock_staff_assess,
    openassessmentblock_self_assess,
    openassessmentblock_save_files_descriptions,
)
from openedx_caliper_tracking.transformers.drag_and_drop_transformers import (
    edx_drag_and_drop_v2_item_dropped,
    edx_drag_and_drop_v2_feedback_closed,
    edx_drag_and_drop_v2_feedback_opened,
)
from openedx_caliper_tracking.transformers.third_party_transformers import (
    edx_googlecomponent_calendar_displayed,
    edx_googlecomponent_document_displayed,
    oppia_exploration_state_changed,
    oppia_exploration_loaded,
    oppia_exploration_completed,
)
from openedx_caliper_tracking.transformers.drag_and_drop_transformers import (
    edx_drag_and_drop_v2_item_dropped,
    edx_drag_and_drop_v2_item_picked_up,
    edx_drag_and_drop_v2_loaded,
)
from openedx_caliper_tracking.transformers.peer_instruction_transformers import (
    ubc_peer_instruction_revised_submitted,
    ubc_peer_instruction_original_submitted,
    ubc_peer_instruction_accessed,
)
from openedx_caliper_tracking.transformers.team_transformers import (
    edx_team_page_viewed,
    edx_team_changed,
    edx_team_learner_added,
    edx_team_searched,
    edx_team_learner_removed,
    edx_team_deleted,
    edx_team_created,
    edx_team_learner_added,
    edx_team_activity_updated,
)
from openedx_caliper_tracking.transformers.cohort_transformers import (
    edx_cohort_user_added,
    edx_cohort_created,
    edx_cohort_user_removed,
    edx_cohort_creation_requested,
    edx_cohort_user_add_requested,
)
from openedx_caliper_tracking.transformers.exam_transformers import (
    edx_special_exam_timed_attempt_created,
    edx_special_exam_timed_attempt_started,
    edx_special_exam_timed_attempt_submitted,
    edx_special_exam_practice_attempt_created,
    edx_special_exam_timed_attempt_created,
    edx_special_exam_timed_attempt_deleted,
    edx_special_exam_time_attempt_ready_to_submit,
    edx_special_exam_timed_created,
    edx_special_exam_proctored_updated,
    edx_special_exam_timed_updated,
    edx_special_exam_proctored_created,
    edx_special_exam_practice_created,
    edx_special_exam_practice_updated,
)
from openedx_caliper_tracking.transformers.user_settings_transformers import (
    edx_user_settings_viewed,
    edx_bi_course_upgrade_sidebarupsell_displayed,
    edx_user_settings_changed,
)
from openedx_caliper_tracking.transformers.xmodule_transformers import (
    xmodule_partitions_assigned_user_to_partition,
)
from openedx_caliper_tracking.transformers.certificate_transformers import (
    edx_certificate_created
)
from openedx_caliper_tracking.transformers.content_library_transformers import (
    edx_librarycontentblock_content_removed,
)
from openedx_caliper_tracking.transformers.course_transfomers import (
    edx_course_home_resume_course_clicked,
    edx_grades_grading_policy_changed
)
from openedx_caliper_tracking.transformers.content_library_transformers import (
    edx_librarycontentblock_content_assigned,
)
from openedx_caliper_tracking.transformers.course_content_completion_transformers import edx_done_toggled
from openedx_caliper_tracking.transformers.certificate_transformers import (
    edx_certificate_evidence_visited,
)

from openedx_caliper_tracking.transformers.certificate_transformers import (
    edx_certificate_shared
)
from openedx_caliper_tracking.transformers.session_transformers import (
    edx_user_login,
    edx_user_logout
)



"""
Utils required in transformers
"""
from dateutil.parser import parse
from django.contrib.auth import get_user_model
from django.conf import settings
from django.core.urlresolvers import reverse
from django.urls.exceptions import NoReverseMatch

UTC_DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%f'


def convert_datetime(current_datetime):
    """
    Convert provided datetime into UTC format
    @param datetime: datetime string.
    :return: UTC formatted datetime string.
    """

    # convert current_datetime to a datetime object if it is string
    if type(current_datetime) in (str, unicode):
        current_datetime = parse(current_datetime)

    utc_offset = current_datetime.utcoffset()
    utc_datetime = current_datetime - utc_offset

    formatted_datetime = '{}{}'.format(
        utc_datetime.strftime(UTC_DATETIME_FORMAT)[:-3], 'Z'
    )
    return formatted_datetime


def get_username_from_user_id(user_id):
    """
    @param : user_id
    :return: username from the given user_id.
    """

    User = get_user_model()
    user = User.objects.get(id=user_id)
    return str(user.username)


def get_user_link_from_username(username):
    try:
        link = '{lms_url}{profile_link}'.format(
            lms_url=settings.LMS_ROOT_URL,
            profile_link=str(reverse(
                'learner_profile',
                kwargs={'username': username}
            ))
        )
    except NoReverseMatch:
        link = '{lms_url}/u/{username}'.format(
            lms_url=settings.LMS_ROOT_URL,
            username=username
        )
    return link


def get_topic_id_from_team_id(team_id):
    """
    :param team_id: extracting from event logs
    :return: topic_id for making team url
    """
    from lms.djangoapps.teams.models import CourseTeam
    user_team = CourseTeam.objects.get(team_id=team_id)

    return user_team.topic_id


def get_team_url_from_team_id(referer, team_id):
    """
    :param referer: extract from event logs
    :param team_id: extract from event logs
    :return: team url
    """
    topic_id = get_topic_id_from_team_id(team_id)
    object_link = '{referrer}#teams/{topic_id}/{team_id}'.format(
        referrer=referer,
        topic_id=topic_id,
        team_id=team_id
    )
    return object_link


def get_certificate_url(user_id, course_id):
    """
    This function takes user_id and course_id as parameters
    makes a reverse url for certificate generated by that user.

    :return: A URI of required certificate.
    """
    certificate_uri = '{lms_url}{certificate_link}'.format(
        lms_url=settings.LMS_ROOT_URL,
        certificate_link=str(reverse(
            'certificates:html_view',
            kwargs={
                'user_id': user_id,
                'course_id': course_id
            }
        ))
    )
    return certificate_uri



from openedx_caliper_tracking import transformers as ctf

"""
Mapping of events to their transformer functions
"""

EVENT_MAPPING = {
    'edx.bookmark.added': ctf.edx_bookmark_added,
    'edx.bookmark.listed': ctf.edx_bookmark_listed,
    'edx.bookmark.accessed': ctf.edx_bookmark_accessed,
    'edx.bookmark.removed': ctf.edx_bookmark_removed,
    'edx.ui.lms.link_clicked': ctf.edx_ui_lms_link_clicked,
    'edx.course.enrollment.activated': ctf.edx_course_enrollment_activated,
    'edx.course.enrollment.deactivated': ctf.edx_course_enrollment_deactivated,
    'edx.course.enrollment.mode_changed':
        ctf.edx_course_enrollment_mode_changed,
    'edx.course.enrollment.upgrade.clicked':
        ctf.edx_course_enrollment_upgrade_clicked,
    'speed_change_video': ctf.edx_video_speed_changed,
    'edx.course.tool.accessed': ctf.edx_course_tool_accessed,
    'edx.forum.response.created': ctf.edx_forum_response_created,
    'problem_show': ctf.problem_show,
    'edx.ui.lms.sequence.next_selected': ctf.edx_ui_lms_sequence_next_selected,
    'edx.ui.lms.sequence.previous_selected':
        ctf.edx_ui_lms_sequence_previous_selected,
    'seq_next': ctf.seq_next,
    'seq_prev': ctf.seq_prev,
    'stop_video': ctf.stop_video,
    'problem_graded': ctf.problem_graded,
    'problem_save': ctf.problem_save,
    'edx.grades.problem.submitted': ctf.edx_grades_problem_submitted,
    'edx.problem.hint.demandhint_displayed':
        ctf.edx_problem_hint_demandhint_displayed,
    'edx.problem.hint.feedback_displayed':
        ctf.edx_problem_hint_feedback_displayed,
    'pause_video': ctf.pause_video,
    'seek_video': ctf.seek_video,
    'load_video': ctf.load_video,
    'show_transcript': ctf.show_transcript,
    'edx.video.closed_captions.shown': ctf.edx_video_closed_captions_shown,
    'edx.video.closed_captions.hidden': ctf.edx_video_closed_captions_hidden,
    'video_show_cc_menu': ctf.video_show_cc_menu,
    'problem_reset': ctf.problem_reset,
    'seq_goto': ctf.seq_goto,
    'problem_rescore': ctf.problem_rescore,
    'edx.forum.thread.viewed': ctf.edx_forum_thread_viewed,
    'save_problem_success': ctf.save_problem_success,
    'play_video': ctf.play_video,
    'hide_transcript': ctf.hide_transcript,
    'video_hide_cc_menu': ctf.video_hide_cc_menu,
    'edx.forum.thread.created': ctf.edx_forum_thread_created,
    'openassessmentblock.get_peer_submission':
        ctf.openassessmentblock_get_peer_submission,
    'openassessmentblock.peer_assess': ctf.openassessmentblock_peer_assess,
    'edx.course.student_notes.viewed': ctf.edx_course_student_notes_viewed,
    'textbook.pdf.page.navigated': ctf.textbook_pdf_page_navigated,
    'edx.forum.comment.created': ctf.edx_forum_comment_created,
    'xblock.poll.view_results': ctf.xblock_poll_view_results,
    'xblock.survey.view_results': ctf.xblock_survey_view_results,
    'openassessment.student_training_assess_example':
        ctf.openassessment_student_training_assess_example,
    'xblock.survey.submitted': ctf.xblock_survey_submitted,
    'problem_check': ctf.problem_check,
    'openassessmentblock.create_submission':
        ctf.openassessmentblock_create_submission,
    'xblock.poll.submitted': ctf.xblock_poll_submitted,
    'edx.course.student_notes.deleted': ctf.edx_course_student_notes_deleted,
    'edx.course.student_notes.edited': ctf.edx_course_student_notes_edited,
    'edx.course.student_notes.added': ctf.edx_course_student_notes_added,
    'openassessmentblock.get_submission_for_staff_grading':
        ctf.openassessmentblock_get_submission_for_staff_grading,
    'textbook.pdf.page.scrolled': ctf.textbook_pdf_page_scrolled,
    'textbook.pdf.zoom.menu.changed': ctf.textbook_pdf_zoom_menu_changed,
    'textbook.pdf.thumbnail.navigated': ctf.textbook_pdf_thumbnail_navigated,
    'textbook.pdf.zoom.buttons.changed': ctf.textbook_pdf_zoom_buttons_changed,
    'book': ctf.book,
    'edx.course.enrollment.upgrade.succeeded':
        ctf.edx_course_enrollment_upgrade_succeeded,
    'edx.course.student_notes.notes_page_viewed':
        ctf.edx_course_student_notes_notes_page_viewed,
    'edx.cohort.user_added': ctf.edx_cohort_user_added,
    'edx.forum.thread.voted': ctf.edx_forum_thread_voted,
    'edx.forum.response.voted': ctf.edx_forum_response_voted,
    'openassessmentblock.submit_feedback_on_assessments':
        ctf.openassessmentblock_submit_feedback_on_assessments,
    'openassessmentblock.save_submission':
        ctf.openassessmentblock_save_submission,
    'edx.googlecomponent.document.displayed':
        ctf.edx_googlecomponent_document_displayed,
    'edx.googlecomponent.calendar.displayed':
        ctf.edx_googlecomponent_calendar_displayed,
    'openassessmentblock.staff_assess': ctf.openassessmentblock_staff_assess,
    'edx.forum.searched': ctf.edx_forum_searched,
    'oppia.exploration.state.changed': ctf.oppia_exploration_state_changed,
    'textbook.pdf.search.executed': ctf.textbook_pdf_search_executed,
    'openassessmentblock.self_assess': ctf.openassessmentblock_self_assess,
    'edx.drag_and_drop_v2.item.dropped': ctf.edx_drag_and_drop_v2_item_dropped,
    'edx.drag_and_drop_v2.item.picked_up':
        ctf.edx_drag_and_drop_v2_item_picked_up,
    'oppia.exploration.loaded': ctf.oppia_exploration_loaded,
    'ubc.peer_instruction.original_submitted':
        ctf.ubc_peer_instruction_original_submitted,
    'ubc.peer_instruction.revised_submitted':
        ctf.ubc_peer_instruction_revised_submitted,
    'edx.drag_and_drop_v2.loaded': ctf.edx_drag_and_drop_v2_loaded,
    'edx.team.changed': ctf.edx_team_changed,
    'edx.course.home.resume_course.clicked':
        ctf.edx_course_home_resume_course_clicked,
    'edx.special_exam.timed.attempt.created':
        ctf.edx_special_exam_timed_attempt_created,
    'edx.special_exam.timed.attempt.started':
        ctf.edx_special_exam_timed_attempt_started,
    'edx.course.student_notes.used_unit_link':
        ctf.edx_course_student_notes_used_unit_link,
    'edx.special_exam.timed.attempt.submitted':
        ctf.edx_special_exam_timed_attempt_submitted,
    'edx.drag_and_drop_v2.feedback.opened':
        ctf.edx_drag_and_drop_v2_feedback_opened,
    'edx.drag_and_drop_v2.feedback.closed':
        ctf.edx_drag_and_drop_v2_feedback_closed,
    'edx.special_exam.timed.attempt.ready_to_submit':
        ctf.edx_special_exam_time_attempt_ready_to_submit,
    'edx.cohort.created': ctf.edx_cohort_created,
    'edx.special_exam.practice.attempt.created':
        ctf.edx_special_exam_practice_attempt_created,
    'edx.cohort.user_removed': ctf.edx_cohort_user_removed,
    'edx.team.page_viewed': ctf.edx_team_page_viewed,
    'edx.grades.problem.state_deleted': ctf.edx_grades_problem_state_deleted,
    'edx.grades.problem.rescored': ctf.edx_grades_problem_rescored,
    'reset_problem_fail': ctf.reset_problem_fail,
    'edx.grades.problem.score_overridden':
        ctf.edx_grades_problem_score_overridden,
    'edx.team.learner_added': ctf.edx_team_learner_added,
    'reset_problem': ctf.reset_problem,
    'showanswer': ctf.showanswer,
    'edx.special_exam.timed.attempt.deleted':
        ctf.edx_special_exam_timed_attempt_deleted,
    'textbook.pdf.outline.toggled': ctf.textbook_pdf_outline_toggled,
    'oppia.exploration.completed': ctf.oppia_exploration_completed,
    'edx.team.searched': ctf.edx_team_searched,
    'textbook.pdf.search.highlight.toggled':
        ctf.textbook_pdf_search_highlight_toggled,
    'textbook.pdf.searchcasesensitivity.toggled':
        ctf.textbook_pdf_searchcasesensitivity_toggled,
    'edx.course.student_notes.searched': ctf.edx_course_student_notes_searched,
    'edx.user.settings.viewed': ctf.edx_user_settings_viewed,
    'textbook.pdf.thumbnails.toggled': ctf.textbook_pdf_thumbnails_toggled,
    'edx.team.deleted': ctf.edx_team_deleted,
    'edx.team.created': ctf.edx_team_created,
    'ubc.peer_instruction.accessed': ctf.ubc_peer_instruction_accessed,
    'problem_check_fail': ctf.problem_check_fail,
    'textbook.pdf.display.scaled': ctf.textbook_pdf_display_scaled,
    'textbook.pdf.chapter.navigated': ctf.textbook_pdf_chapter_navigated,
    'textbook.pdf.search.navigatednext':
        ctf.textbook_pdf_search_navigatednext,
    'edx.certificate.created': ctf.edx_certificate_created,
    'edx.librarycontentblock.content.removed':
        ctf.edx_librarycontentblock_content_removed,
    'edx.user.settings.changed': ctf.edx_user_settings_changed,
    'xmodule.partitions.assigned_user_to_partition':
        ctf.xmodule_partitions_assigned_user_to_partition,
    'openassessmentblock.save_files_descriptions':
        ctf.openassessmentblock_save_files_descriptions,
    'xblock.split_test.child_render': ctf.xblock_split_test_child_render,
    'edx.librarycontentblock.content.assigned':
        ctf.edx_librarycontentblock_content_assigned,
    'edx.bi.course.upgrade.sidebarupsell.displayed':
        ctf.edx_bi_course_upgrade_sidebarupsell_displayed,
    'save_problem_fail': ctf.save_problem_fail,
    'edx.team.learner_removed': ctf.edx_team_learner_removed,
    'edx.team.activity_updated': ctf.edx_team_activity_updated,
    'edx.special_exam.proctored.created': ctf.edx_special_exam_proctored_created,
    'edx.special_exam.practice.created': ctf.edx_special_exam_practice_created,
    'edx.done.toggled': ctf.edx_done_toggled,
    'edx.special_exam.practice.updated': ctf.edx_special_exam_practice_updated,
    'edx.special_exam.timed.updated': ctf.edx_special_exam_timed_updated,
    'edx.special_exam.proctored.updated': ctf.edx_special_exam_proctored_updated,
    'edx.special_exam.timed.created': ctf.edx_special_exam_timed_created,
    'edx.grades.grading_policy_changed': ctf.edx_grades_grading_policy_changed,
    'page_close': ctf.page_close,
    'edx.certificate.evidence_visited': ctf.edx_certificate_evidence_visited,
    'edx.certificate.shared': ctf.edx_certificate_shared,
    'edx.cohort.creation_requested': ctf.edx_cohort_creation_requested,
    'edx.cohort.user_add_requested': ctf.edx_cohort_user_add_requested,
    'edx.user.login': ctf.edx_user_login,
    'edx.user.logout': ctf.edx_user_logout,
}



"""
Base module containing generic caliper transformer class
"""
import uuid
import json

from openedx_caliper_tracking import utils

CALIPER_EVENT_CONTEXT = 'http://purl.imsglobal.org/ctx/caliper/v1p1'


def base_transformer(event):
    """Transforms event into caliper format
    @param event: unprocessed event dict
    """
    caliper_event = {}

    _add_generic_fields(event, caliper_event)
    _add_actor_info(event, caliper_event)
    _add_referrer(event, caliper_event)
    _add_extensions(event, caliper_event)

    return caliper_event


def page_view_transformer(event):
    """Transforms page view events into caliper format

    @param event: non-caliperized event dict
    @return : caliperized event dict
    """

    caliper_event = base_transformer(event)

    caliper_event['actor']['type'] = 'Person'
    caliper_event['referrer']['type'] = 'WebPage'
    caliper_event['type'] = 'NavigationEvent'
    caliper_event['action'] = 'NavigatedTo'
    caliper_event['object'] = {
        'id': event.get('referer'),
        'type': 'WebPage',
        'extensions': event.get('event')
    }
    caliper_event['extensions']['extra_fields'].update(event.get('context'))
    caliper_event_str = json.dumps(caliper_event)
    return caliper_event_str


def _add_generic_fields(event, caliper_event):
    """
    Adds all of the generic fields to the caliper_event object.

    @param event: unprocessed event dict
    @param caliper_event: caliper event dict
    """
    caliper_event.update({
        '@context': CALIPER_EVENT_CONTEXT,
        'id': uuid.uuid4().urn,
        'eventTime': utils.convert_datetime(event.get('time'))
    })


def _add_actor_info(event, caliper_event):
    """
    Adds all generic information related to `actor`

    @param event: unprocessed event dict
    @param caliper_event: caliper event dict
    """
    caliper_event['actor'] = {}
    user_profile_link = utils.get_user_link_from_username(event.get('username'))
    caliper_event['actor'].update({
        'id': user_profile_link,
    })


def _add_extensions(event, caliper_event):
    """
    A map of additional attributes not defined by the model MAY be
    specified for a more concise representation of the Event.

    @param event: unprocessed event dict
    @param caliper_event: caliper event dict
    """
    caliper_event['extensions'] = {}
    caliper_event['extensions']['extra_fields'] = {
        'agent': event.get('agent'),
        'event_type': event.get('event_type'),
        'event_source': event.get('event_source'),
        'host': event.get('host'),
        'org_id': event['context'].get('org_id'),
        'path': event['context'].get('path'),
        'session': event.get('session'),
        'user_id': event['context'].get('user_id'),
        'accept_language': event.get('accept_language'),
        'page': event.get('page'),
    }


def _add_referrer(event, caliper_event):
    """
    Adds information of an Entity that represents the referring context.

    @param event: unprocessed event dict
    @param caliper_event: caliper event dict
    """
    caliper_event['referrer'] = {
        'id': event.get('referer')
    }



"""
Transformer for all xmodule events
"""

from django.conf import settings


def xmodule_partitions_assigned_user_to_partition(current_event, caliper_event):
    """
    When a student views a module that is set up to test different child modules,
    the server checks the user_api_usercoursetag table for the student assignment
    to the relevant partition, and to a group for that partition.

    The partition ID is the user_api_usercoursetag.key.

    The group ID is the user_api_usercoursetag.value.

    :param current_event: default event log generated.
    :param caliper_event: caliper_event log having some basic attributes.
    :return: updated caliper_event.
    """
    caliper_object = {
        'id': current_event['referer'],
        'member': {
            'id': caliper_event['actor']['id'],
            'name': current_event['username'],
            'type': 'Person',
            'extensions': {
                'user_id': caliper_event['extensions']['extra_fields'].pop('user_id'),
            }
        },
        'organization': {
            'extensions': current_event['event'],
            'id': current_event['referer'],
            'type': 'Group'
        },
        'type': 'Membership'
    }

    caliper_event.update({
        'type': 'Event',
        'action': 'Linked',
        'object': caliper_object,
    })

    caliper_event['actor'].update({
        'id': settings.LMS_ROOT_URL,
        'type': 'SoftwareApplication',
    })

    caliper_event['referrer'].update({
        'type': 'WebPage'
    })

    caliper_event['extensions']['extra_fields'].update({
        'ip': current_event['ip'],
        'course_id': current_event['context']['course_id'],
        'course_user_tags': current_event['context']['course_user_tags']
    })

    return caliper_event



"""
All settings for Caliper Tracking app.
"""

OPENEDX_CALIPER_TRACKING_PROCESSOR = 'openedx_caliper_tracking.processor.CaliperProcessor'

OPENEDX_CALIPER_TRACKING_BACKENDS = {
    'logger': {
        'ENGINE': OPENEDX_CALIPER_TRACKING_PROCESSOR,
        'OPTIONS': {
            'name': 'tracking'
        }
    }
}

CALIPER_AUTH = {
    'CALIPER_TESTS_API_URL': '',
    'CALIPER_TESTS_API_TOKEN': '',
}
